package com.persagy.dptool;

import com.persagy.dptool.dto.PhysicalObject;
import javafx.concurrent.Task;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.control.*;
import javafx.scene.layout.BorderPane;
import javafx.stage.DirectoryChooser;
import javafx.stage.FileChooser;

import java.io.File;
import java.net.URL;
import java.util.Date;
import java.util.ResourceBundle;

public class PrimaryController implements Initializable {
    @FXML
    private BorderPane paneRoot;
    @FXML
    private TabPane paneTab;
    /** 配置文件目录 */
    @FXML
    public TextField txfDir;
    @FXML
    public TextField txfDirJson;
    @FXML
    public Button btnSelectDir;
    @FXML
    public TextField txfProject;
    @FXML
    public TextField txfCheckResultFile;
    @FXML
    public Button btnSelectDirJson;
    @FXML
    public Button btnCheck;
    @FXML
    public Button btnCheckJson;
    /** 校验结果文本域 */
    @FXML
    public TextArea txaContent;
    @FXML
    public TextArea txaContentJson;
    @FXML
    public ProgressIndicator piState;
    /** 底部状态栏 */
    @FXML
    public Label lblState;

    private static File tableCheckResultFile;

    @Override
    public void initialize(URL location, ResourceBundle resources) {
        piState.setProgress(-1);
        piState.setVisible(false);
        lblState.setText("");

        StringBuilder tableNameSBStr = new StringBuilder();
        for (int i = 0; i < ConstantData.tablesNameList.size(); i++) {
            tableNameSBStr.append("  " + (i+1) + ". " + ConstantData.tablesNameList.get(i) + "\n");
        }
        txaContentJson.setText("目前只能校验HBase数据库中，物理世界的以下数据表数据：\n" + tableNameSBStr);

        String checkResultFileName = "checkResult" + new Date().getTime() + ".txt";
        txfCheckResultFile.setText(PrimaryController.class.getClassLoader().getResource("").toString() + checkResultFileName);

        // ----
        txfProject.setText("Pj3101040001");
        txfDirJson.setText("D:\\var\\tableJson");
    }

    /**
     * 选择配置文件目录
     * @param e
     */
    public void selectDir(ActionEvent e) {
        DirectoryChooser dirChooser = new DirectoryChooser();
        dirChooser.setTitle("选择配置文件目录");

        File folder = dirChooser.showDialog(paneRoot.getScene().getWindow());

        if(null != folder) {
            txfDir.setText(folder.getAbsolutePath());
        }
    }

    public void selectTxtFile(ActionEvent e) {
        FileChooser fileChooser = new FileChooser();
        fileChooser.setTitle("指定存放结果文件");

        fileChooser.getExtensionFilters().addAll(new FileChooser.ExtensionFilter("Text", "*.txt"));

        File file = fileChooser.showSaveDialog(paneRoot.getScene().getWindow());

        if(null != file) {
            txfCheckResultFile.setText(file.getAbsolutePath());
        }
    }

    /**
     * 选择配置文件目录
     * @param e
     */
    public void selectDirJson(ActionEvent e) {
        DirectoryChooser dirChooser = new DirectoryChooser();
        dirChooser.setTitle("选择json文件目录");

        File folder = dirChooser.showDialog(paneRoot.getScene().getWindow());

        if(null != folder) {
            txfDirJson.setText(folder.getAbsolutePath());
        }
    }

    /**
     * 检查数据平台配置文件
     * @param e
     */
    public void checkConfig(ActionEvent e) {
        File configFile = getFile(txfDir.getText(), "config.properties");
        File propertyFile = getFile(txfDir.getText(), "property");

        if(null == configFile && propertyFile == null) {
            txaContent.setText("未找到需要校验的config.properties文件和property数据字典文件夹，请确保所选目录是否正确。");
            return;
        }

        Task<Boolean> task = TaskFactory.checkConfig(this, configFile, propertyFile);
        new Thread(task).start();
    }

    /**
     * 检查物理世界数据表json文件
     * @param e
     */
    public void checkJsonFile(ActionEvent e) {
        txaContentJson.setText("");
        String projectId = txfProject.getText();
        PhysicalObject physicalObject = null;
        if(projectId != null && projectId.trim().length() == 12 && projectId.startsWith("Pj")) {
            physicalObject = new PhysicalObject(projectId, projectId.substring(2));
        }else {
            txaContentJson.setText("项目id不合法！");
            txfProject.requestFocus();
            return;
        }

        String logFilePath = txfCheckResultFile.getText();
        if(logFilePath == null || logFilePath.trim().length() < 0) {
            txaContentJson.setText("请先指定存放结果文件！");
            txfCheckResultFile.requestFocus();
            return;
        }
        File logFile = new File(logFilePath);
        try {
            if(!logFile.exists()) {
                logFile.createNewFile();
            }

            if(!logFile.exists() || !logFile.isFile()) {
                txaContentJson.setText("存放结果文件无法访问，请重新指定或手动创建！");
                txfCheckResultFile.requestFocus();
                return;
            }
        }catch (Exception ex) {
            logFile = null;
        }

        if(logFile == null) {
            txaContentJson.setText("存放结果文件不存在，请手动创建并指定！");
            txfCheckResultFile.requestFocus();
            return;
        }else {
            tableCheckResultFile = logFile;
        }

        File jsonFileDir = null;
        String filePath = txfDirJson.getText();

        if(null != filePath) {
            filePath = filePath.trim();
            jsonFileDir = new File(filePath);
            if(!jsonFileDir.exists() || jsonFileDir.isFile()) {
                txaContentJson.setText("目录地址不合法！");
                return;
            }
        }else {
            txaContentJson.setText("目录地址不合法！");
        }

        Task<Boolean> task = TaskFactory.checkTableJsonFiles(this, physicalObject, jsonFileDir);
        new Thread(task).start();
    }

    /**
     * 根据文件路径获取文件实例
     * @param dirPath
     * @return
     */
    private File getFile(String dirPath, String fileName) {
        File file = null;

        if(dirPath != null) {
            dirPath = dirPath.trim() + File.separator + fileName;
            file = new File(dirPath);
            if(!file.exists()) {
                file = null;
            }
        }
        return file;
    }

    /**
     * 设置tab页签中的控件是否可用
     * @param disable true-可用
     * @param typeIndex 从1开始
     */
    public void setDisable(boolean disable, int typeIndex) {
        if(1 == typeIndex) {
            paneTab.setDisable(disable);
        }else if(2 == typeIndex){
            paneTab.setDisable(disable);
        }

    }
}
